@@ -653,11 +653,58 @@ int bladerf_set_rational_sample_rate(struct bladerf *dev,
                                      struct bladerf_rational_rate *actual)
 {
     int status;
+    bladerf_feature feature = dev->feature;
+
     MUTEX_LOCK(&dev->lock);
-
     status = dev->board->set_rational_sample_rate(dev, ch, rate, actual);
-
     MUTEX_UNLOCK(&dev->lock);
+
+    /*****************************************************
+      Register config for OVERSAMPLE operation
+
+      Sample rate assignments clear previous register
+      values. We must reassign for every set_samplerate().
+
+      Note: bladerf_set_rfic_register is mutex locked. Must
+            be placed outside of a mutex lock like above.
+    *******************************************************/
+    if ((feature & BLADERF_FEATURE_OVERSAMPLE)) {
+        bladerf_set_rfic_register(dev,0x003,0x54); // OC Register
+
+        /* TX Register Assignments */
+        bladerf_set_rfic_register(dev,0x02,0xc0);  // TX Enable and Filter Control
+        bladerf_set_rfic_register(dev,0xc2,0x9f);  // TX BBF R1
+        bladerf_set_rfic_register(dev,0xc3,0x9f);  // TX baseband filter R2
+        bladerf_set_rfic_register(dev,0xc4,0x9f);  // TX baseband filter R3
+        bladerf_set_rfic_register(dev,0xc5,0x9f);  // TX baseband filter R4
+        bladerf_set_rfic_register(dev,0xc6,0x9f);  // TX baseband filter real pole word
+        bladerf_set_rfic_register(dev,0xc7,0x00);  // TX baseband filter C1
+        bladerf_set_rfic_register(dev,0xc8,0x00);  // TX baseband filter C2
+        bladerf_set_rfic_register(dev,0xc9,0x00);  // TX baseband filter real pole word
+
+        /* RX Register Assignments */
+        // Gain and calibration
+        bladerf_set_rfic_register(dev,0x1e0,0xBF);
+        bladerf_set_rfic_register(dev,0x1e4,0xFF);
+        bladerf_set_rfic_register(dev,0x1f2,0xFF);
+        // bladerf_set_rfic_register(dev,0x1e6,0x87); // Causes gr-osmosdr to freak out
+
+        // Miller and BBF caps
+        bladerf_set_rfic_register(dev,0x1e7,0x00);
+        bladerf_set_rfic_register(dev,0x1e8,0x00);
+        bladerf_set_rfic_register(dev,0x1e9,0x00);
+        bladerf_set_rfic_register(dev,0x1ea,0x00);
+        bladerf_set_rfic_register(dev,0x1eb,0x00);
+        bladerf_set_rfic_register(dev,0x1ec,0x00);
+        bladerf_set_rfic_register(dev,0x1ed,0x00);
+        bladerf_set_rfic_register(dev,0x1ee,0x00);
+        bladerf_set_rfic_register(dev,0x1ef,0x00);
+        bladerf_set_rfic_register(dev,0x1e0,0xBF);
+
+        // BIST and Data Port Test Config [D1:D0] "Must be 2’b00"
+        bladerf_set_rfic_register(dev,0x3f6,0x03);
+    }
+
     return status;
 }
 
@@ -963,6 +1010,13 @@ int bladerf_init_stream(struct bladerf_stream **stream,
     int status;
     MUTEX_LOCK(&dev->lock);
 
+    if (format == BLADERF_FORMAT_SC8_Q7 || format == BLADERF_FORMAT_SC8_Q7_META) {
+        if (strcmp(bladerf_get_board_name(dev), "bladerf2") != 0) {
+            log_error("bladeRF 2.0 required for 8bit format\n");
+            return BLADERF_ERR_UNSUPPORTED;
+        }
+    }
+
     status = dev->board->init_stream(stream, dev, callback, buffers,
                                      num_buffers, format, samples_per_buffer,
                                      num_transfers, data);
@@ -1033,6 +1087,13 @@ int bladerf_sync_config(struct bladerf *dev,
     int status;
     MUTEX_LOCK(&dev->lock);
 
+    if (format == BLADERF_FORMAT_SC8_Q7 || format == BLADERF_FORMAT_SC8_Q7_META) {
+        if (strcmp(bladerf_get_board_name(dev), "bladerf2") != 0) {
+            log_error("bladeRF 2.0 required for 8bit format\n");
+            return BLADERF_ERR_UNSUPPORTED;
+        }
+    }
+
     status =
         dev->board->sync_config(dev, layout, format, num_buffers, buffer_size,
                                 num_transfers, stream_timeout);
@@ -1891,3 +1952,49 @@ int bladerf_xb300_get_output_power(struct bladerf *dev, float *val)
     MUTEX_UNLOCK(&dev->lock);
     return status;
 }
+
+/******************************************************************************/
+/* Features */
+/******************************************************************************/
+
+int bladerf_enable_feature(struct bladerf *dev, bladerf_feature feature, bool enable)
+{
+    int status;
+    MUTEX_LOCK(&dev->lock);
+
+    status = 0;
+
+    if(feature == BLADERF_FEATURE_DEFAULT) {
+        dev->feature = 0;
+    } else {
+        if(feature == BLADERF_FEATURE_OVERSAMPLE) {
+            if (strcmp(bladerf_get_board_name(dev), "bladerf2") != 0) {
+                log_error("BladeRF2 required for OVERSAMPLE feature\n");
+                status = BLADERF_ERR_UNSUPPORTED;
+            }
+        } else {
+            /* Unknown / Unsupported feature */
+            status = BLADERF_ERR_UNSUPPORTED;
+        }
+
+        if (status == 0) {
+            if (enable) {
+                dev->feature |= feature;
+            } else {
+                dev->feature &= ~feature;
+            }
+        }
+    }
+
+    MUTEX_UNLOCK(&dev->lock);
+    return status;
+}
+
+int bladerf_get_feature(struct bladerf *dev, bladerf_feature* feature)
+{
+    MUTEX_LOCK(&dev->lock);
+    *feature = dev->feature;
+    MUTEX_UNLOCK(&dev->lock);
+
+    return 0;
+}
\ No newline at end of file
